from collections.abc import Generator
from typing import TypeAlias

import pytest
from vodozemac import (
    Account,
    AnyOlmMessage,
    DecodeException,
    Session,
    PickleException,
    PreKeyMessage
)


@pytest.fixture(scope="module")
def alice() -> Account:
    return Account()

@pytest.fixture(scope="module")
def bob() -> Account:
    return Account()

SessionGenerator: TypeAlias = Generator[Session]

@pytest.fixture
def alice_session_gen(alice: Account, bob: Account) -> SessionGenerator:
    def session_generator() -> SessionGenerator:
        while True:
            bob.generate_one_time_keys(1)
            identity_key = bob.curve25519_key
            one_time_key = next(iter(bob.one_time_keys.values()))
            yield alice.create_outbound_session(identity_key, one_time_key)
    return session_generator()

@pytest.fixture
def alice_session(alice_session_gen: SessionGenerator) -> Session:
    return next(alice_session_gen)

def test_create(alice_session_gen: SessionGenerator):
    session1, session2 = next(alice_session_gen), next(alice_session_gen)
    assert session1.session_id != session2.session_id
    for session in (session1, session2):
        assert isinstance(session, Session)
        assert isinstance(session.session_id, str)

def test_clear(alice_session: Session):
    del alice_session

def test_pickle(alice_session: Session, pickle_key: bytes):
    unpickled = Session.from_pickle(alice_session.pickle(pickle_key), pickle_key)
    assert unpickled.session_id == alice_session.session_id

def test_wrong_pickle_key(alice_session: Session, pickle_key: bytes):
    pickle = alice_session.pickle(pickle_key)
    with pytest.raises(PickleException):
        Session.from_pickle(pickle, b"Definitely wrong key")

def test_invalid_pickle(pickle_key: bytes):
    with pytest.raises(PickleException):
        Session.from_pickle("", pickle_key)

def test_two_messages(alice: Account, bob: Account, alice_session: Session):
    alice_plaintext = b"It's a secret to everybody"
    alice_message = alice_session.encrypt(alice_plaintext).to_pre_key()

    assert isinstance(alice_message, PreKeyMessage)

    bob_session, alice_decrypted = bob.create_inbound_session(alice.curve25519_key, alice_message)
    assert alice_plaintext == alice_decrypted

    bob_plaintext = b"Grumble, Grumble"
    bob_message = bob_session.encrypt(bob_plaintext)

    assert bob_plaintext == alice_session.decrypt(bob_message)

def test_empty_message():
    with pytest.raises(DecodeException):
        AnyOlmMessage.from_parts(0, b"x")

def test_matches(alice: Account, bob: Account, alice_session_gen: SessionGenerator):
    alice_plaintext = b"It's a secret to everybody"
    alice_session1, alice_session2 = next(alice_session_gen), next(alice_session_gen)
    alice_message1 = alice_session1.encrypt(alice_plaintext).to_pre_key()

    bob_session, alice_decrypted = bob.create_inbound_session(alice.curve25519_key, alice_message1)
    assert alice_plaintext == alice_decrypted

    alice_message2 = alice_session2.encrypt(alice_plaintext).to_pre_key()
    assert bob_session.session_matches(alice_message2) is False

def test_does_not_match(alice: Account, bob: Account, alice_session: Session):
    alice_plaintext = b"It's a secret to everybody"
    alice_message1 = alice_session.encrypt(alice_plaintext).to_pre_key()

    bob_session, alice_decrypted = bob.create_inbound_session(alice.curve25519_key, alice_message1)
    assert alice_plaintext == alice_decrypted

    alice_message2 = alice_session.encrypt(b"Hey! Listen!").to_pre_key()
    assert bob_session.session_matches(alice_message2) is True

def test_invalid(alice_session_gen: SessionGenerator):
    session1, session2 = next(alice_session_gen), next(alice_session_gen)
    message = session1.encrypt(b"It's a secret to everybody").to_pre_key()

    assert not session2.session_matches(message)

def test_message_to_parts(alice: Account, bob: Account, alice_session: Session):
    alice_plaintext = b"It's a secret to everybody"
    encrypted = alice_session.encrypt(alice_plaintext)

    alice_message = encrypted.to_pre_key()
    alice_message_from_parts = AnyOlmMessage.from_parts(*encrypted.to_parts()).to_pre_key()

    assert alice_message.session_id() == alice_message_from_parts.session_id()

    bob_session, alice_decrypted = bob.create_inbound_session(alice.curve25519_key, alice_message)
    assert alice_plaintext == alice_decrypted



import contextlib
from .vodozemac import *  # type: ignore

# Define all available exports for better IDE support
__all__ = [
    # Classes
    "Account",
    "Session", 
    "AnyOlmMessage",
    "PreKeyMessage",
    "Sas",
    "EstablishedSas", 
    "GroupSession",
    "InboundGroupSession",
    "SessionKey",
    "ExportedSessionKey", 
    "MegolmMessage",
    "Ed25519PublicKey",
    "Ed25519Signature",
    "Curve25519PublicKey", 
    "Curve25519SecretKey",
    "PkDecryption",
    "PkEncryption",
    "Message",
    # Exceptions
    "KeyException",
    "SignatureException", 
    "DecodeException",
    "LibolmPickleException",
    "SessionKeyDecodeException",
    "PickleException",
    "SessionCreationException",
    "SasException", 
    "OlmDecryptionException",
    "MegolmDecryptionException",
    "PkInvalidKeySizeException",
    "PkDecodeException",
]

with contextlib.suppress(ImportError):
    from . import vodozemac  # type: ignore
    __doc__ = vodozemac.__doc__
    if hasattr(vodozemac, "__all__"):
        __all__ = vodozemac.__all__ 


from typing import Final

import pytest
from vodozemac import Sas, EstablishedSas
from vodozemac.vodozemac import Curve25519PublicKey

EXTRA_INFO: Final[str] = "extra_info"

@pytest.fixture
def alice_sas() -> Sas:
    return Sas()

@pytest.fixture
def bob_sas() -> Sas:
    return Sas()

@pytest.fixture
def alice_established_sas(alice_sas: Sas, bob_sas: Sas) -> EstablishedSas:
    return alice_sas.diffie_hellman(bob_sas.public_key)

@pytest.fixture
def bob_established_sas(alice_sas: Sas, bob_sas: Sas) -> EstablishedSas:
    return bob_sas.diffie_hellman(alice_sas.public_key)

def test_creation(alice_sas: Sas, alice_established_sas):
    assert isinstance(alice_sas.public_key, Curve25519PublicKey)
    assert isinstance(alice_established_sas, EstablishedSas)

def test_bytes_generating(alice_sas: Sas, bob_sas: Sas):
    alice_bytes = alice_sas.diffie_hellman(bob_sas.public_key).bytes(info=EXTRA_INFO)
    bob_bytes = bob_sas.diffie_hellman(alice_sas.public_key).bytes(info=EXTRA_INFO)

    assert alice_bytes.emoji_indices == bob_bytes.emoji_indices
    assert alice_bytes.decimals == bob_bytes.decimals

def test_mac_generating(alice_established_sas: EstablishedSas, bob_established_sas: EstablishedSas):
    message = "Test message"
    alice_mac = alice_established_sas.calculate_mac(message, EXTRA_INFO)
    bob_mac = bob_established_sas.calculate_mac(message, EXTRA_INFO)

    assert alice_established_sas.verify_mac(message, EXTRA_INFO, bob_mac) is None
    assert bob_established_sas.verify_mac(message, EXTRA_INFO, alice_mac) is None

    assert alice_mac == bob_mac



import pytest
from hypothesis import given
from vodozemac import Account, PickleException, SignatureException, Ed25519PublicKey, Curve25519PublicKey

@pytest.fixture(scope="module")
def account() -> Account:
    return Account()

def test_creation(account: Account):
    assert isinstance(account.ed25519_key, Ed25519PublicKey)
    assert isinstance(account.curve25519_key, Curve25519PublicKey)
    assert isinstance(account.max_number_of_one_time_keys, int)

def test_generate_and_publish_one_time_keys(account: Account):
    assert len(account.one_time_keys) == 0
    account.generate_one_time_keys(10)
    assert len(account.one_time_keys) == 10
    account.mark_keys_as_published()
    assert not account.one_time_keys

def test_pickling(account: Account, pickle_key: bytes):
    pickle = account.pickle(pickle_key)
    unpickled = Account.from_pickle(pickle, pickle_key)
    assert account.ed25519_key == unpickled.ed25519_key
    assert account.curve25519_key == unpickled.curve25519_key
    assert account.one_time_keys == unpickled.one_time_keys

def test_libolm_pickling():
    pickle = (
        "3wpPcPT4xsRYCYF34NcnozxE5bN2E6qwBXQYuoovt/TX//8Dnd8gaKsxN9En/"
        "7Hkh5XemuGUo3dXHVTl76G2pjf9ehfryhITMbeBrE/XuxmNvS2aB9KU4mOKXl"
        "AWhCEsE7JW9fUkRhHWWkFwTvSC3eDthd6eNx3VKZlmGR270vIpIG5/Ho4YK9/"
        "03lPGpil0cuEuGTTjKHXGRu9kpnQe99QGCB4KBuP5IJjFeWbtSgJ4ZrajZdlTew"
    )

    unpickled = Account.from_libolm_pickle(pickle, b"It's a secret to everybody")

    assert unpickled.ed25519_key.to_base64() == "MEQCwaTE/gcrHaxwv06WEVy5xDA30FboFzCAtYhzmoc"

def test_invalid_pickle(pickle_key: bytes):
    with pytest.raises(PickleException):
        Account.from_pickle("", pickle_key)

@given(message=...)
def test_signing(account: Account, message: bytes):
    signature = account.sign(message)
    account.ed25519_key.verify_signature(message, signature)
    with pytest.raises(SignatureException):
        account.ed25519_key.verify_signature(b"This should fail", signature)



"""Type stubs for vodozemac - Python bindings for the vodozemac Rust library."""

from typing import Optional, Dict, Any, Tuple, List
from typing_extensions import Self

__all__ = [
    "Account", "Session", "AnyOlmMessage", "PreKeyMessage", "Sas", "EstablishedSas", 
    "GroupSession", "InboundGroupSession", "SessionKey", "ExportedSessionKey", 
    "MegolmMessage", "Ed25519PublicKey", "Ed25519Signature", "Curve25519PublicKey", 
    "Curve25519SecretKey", "PkDecryption", "PkEncryption", "Message", "KeyException",
    "SignatureException", "DecodeException", "LibolmPickleException", "SessionKeyDecodeException",
    "PickleException", "SessionCreationException", "SasException", "OlmDecryptionException", 
    "MegolmDecryptionException", "PkInvalidKeySizeException", "PkDecodeException"
]

# Exceptions
class KeyException(ValueError): ...
class SignatureException(ValueError): ...
class DecodeException(ValueError): ...
class LibolmPickleException(ValueError): ...
class SessionKeyDecodeException(ValueError): ...
class PickleException(ValueError): ...
class SessionCreationException(ValueError): ...
class SasException(ValueError): ...
class OlmDecryptionException(ValueError): ...
class MegolmDecryptionException(ValueError): ...
class PkInvalidKeySizeException(ValueError): ...
class PkDecodeException(ValueError): ...

# Key Types
class Ed25519PublicKey:
    """An Ed25519 public key."""
    
    @classmethod
    def from_base64(cls, key: str) -> Ed25519PublicKey: ...
    def to_base64(self) -> str: ...
    def verify_signature(self, message: bytes, signature: Ed25519Signature) -> None: ...
    def __eq__(self, other: object) -> bool: ...

class Ed25519Signature:
    """An Ed25519 signature."""
    
    @classmethod
    def from_base64(cls, signature: str) -> Ed25519Signature: ...
    def to_base64(self) -> str: ...

class Curve25519PublicKey:
    """A Curve25519 public key."""
    
    @classmethod
    def from_base64(cls, key: str) -> Curve25519PublicKey: ...
    @classmethod
    def from_bytes(cls, bytes: bytes) -> Curve25519PublicKey: ...
    def to_base64(self) -> str: ...
    def to_bytes(self) -> bytes: ...
    def __eq__(self, other: object) -> bool: ...

class Curve25519SecretKey:
    """A Curve25519 secret key."""
    
    def __init__(self) -> None: ...
    @classmethod
    def from_base64(cls, key: str) -> Curve25519SecretKey: ...
    @classmethod
    def from_bytes(cls, bytes: bytes) -> Curve25519SecretKey: ...
    def to_base64(self) -> str: ...
    def to_bytes(self) -> bytes: ...
    def public_key(self) -> Curve25519PublicKey: ...

# Session Keys
class SessionKey:
    """A Megolm session key."""
    
    def __init__(self, session_key: str) -> None: ...
    def to_base64(self) -> str: ...

class ExportedSessionKey:
    """An exported Megolm session key."""
    
    def __init__(self, session_key: str) -> None: ...
    def to_base64(self) -> str: ...

# Messages
class AnyOlmMessage:
    """A message encrypted using the Olm protocol."""
    
    @classmethod
    def pre_key(cls, message: bytes) -> AnyOlmMessage: ...
    @classmethod
    def normal(cls, message: bytes) -> AnyOlmMessage: ...
    @classmethod
    def from_parts(cls, message_type: int, ciphertext: bytes) -> AnyOlmMessage: ...
    def to_pre_key(self) -> Optional[PreKeyMessage]: ...
    def to_parts(self) -> Tuple[int, bytes]: ...

class PreKeyMessage:
    """A pre-key message for the Olm protocol."""
    
    @classmethod
    def from_base64(cls, message: str) -> PreKeyMessage: ...
    def to_any(self) -> AnyOlmMessage: ...
    def session_id(self) -> str: ...

class MegolmMessage:
    """A Megolm encrypted message."""
    
    @classmethod
    def from_base64(cls, message: str) -> MegolmMessage: ...
    @classmethod
    def from_bytes(cls, message: bytes) -> MegolmMessage: ...
    def to_base64(self) -> str: ...
    def to_bytes(self) -> bytes: ...
    def message_index(self) -> int: ...
    def signature(self) -> Ed25519Signature: ...

# Account and Session
class Account:
    """An Olm account."""
    
    def __init__(self) -> None: ...
    @classmethod
    def from_pickle(cls, pickle: str, pickle_key: bytes) -> Account: ...
    @classmethod
    def from_libolm_pickle(cls, pickle: str, pickle_key: bytes) -> Account: ...
    def pickle(self, pickle_key: bytes) -> str: ...
    
    @property
    def ed25519_key(self) -> Ed25519PublicKey: ...
    @property
    def curve25519_key(self) -> Curve25519PublicKey: ...
    @property
    def one_time_keys(self) -> Dict[str, Curve25519PublicKey]: ...
    @property
    def max_number_of_one_time_keys(self) -> int: ...
    
    def sign(self, message: bytes) -> Ed25519Signature: ...
    def generate_one_time_keys(self, count: int) -> None: ...
    def mark_keys_as_published(self) -> None: ...
    def create_outbound_session(self, identity_key: Curve25519PublicKey, one_time_key: Curve25519PublicKey) -> Session: ...
    def create_inbound_session(self, sender_key: Curve25519PublicKey, message: PreKeyMessage) -> Tuple[Session, bytes]: ...

class Session:
    """An Olm session."""
    
    @classmethod
    def from_pickle(cls, pickle: str, pickle_key: bytes) -> Session: ...
    @classmethod
    def from_libolm_pickle(cls, pickle: str, pickle_key: bytes) -> Session: ...
    
    @property
    def session_id(self) -> str: ...
    
    def pickle(self, pickle_key: bytes) -> str: ...
    def session_matches(self, message: PreKeyMessage) -> bool: ...
    def encrypt(self, plaintext: bytes) -> AnyOlmMessage: ...
    def decrypt(self, message: AnyOlmMessage) -> bytes: ...

# Group Sessions
class GroupSession:
    """An outbound Megolm group session."""
    
    def __init__(self) -> None: ...
    @classmethod
    def from_pickle(cls, pickle: str, pickle_key: bytes) -> GroupSession: ...
    
    @property
    def session_id(self) -> str: ...
    @property
    def message_index(self) -> int: ...
    @property
    def session_key(self) -> SessionKey: ...
    
    def encrypt(self, plaintext: bytes) -> MegolmMessage: ...
    def pickle(self, pickle_key: bytes) -> str: ...

class DecryptedMessage:
    """A decrypted Megolm message."""
    plaintext: bytes
    message_index: int

class InboundGroupSession:
    """An inbound Megolm group session."""
    
    def __init__(self, session_key: SessionKey) -> None: ...
    @classmethod
    def import_session(cls, session_key: ExportedSessionKey) -> InboundGroupSession: ...
    @classmethod
    def from_pickle(cls, pickle: str, pickle_key: bytes) -> InboundGroupSession: ...
    
    @property
    def session_id(self) -> str: ...
    @property
    def first_known_index(self) -> int: ...
    
    def export_at(self, index: int) -> Optional[ExportedSessionKey]: ...
    def decrypt(self, message: MegolmMessage) -> DecryptedMessage: ...
    def pickle(self, pickle_key: bytes) -> str: ...

# SAS (Short Authentication String)
class Sas:
    """A SAS object for interactive key verification."""
    
    def __init__(self) -> None: ...
    @property
    def public_key(self) -> Curve25519PublicKey: ...
    def diffie_hellman(self, key: Curve25519PublicKey) -> EstablishedSas: ...

class EstablishedSas:
    """An established SAS session."""
    
    def bytes(self, info: str) -> SasBytes: ...
    def calculate_mac_invalid_base64(self, input: str, info: str) -> str: ...
    def calculate_mac(self, input: str, info: str) -> str: ...
    def verify_mac(self, input: str, info: str, tag: str) -> None: ...

class SasBytes:
    """SAS bytes for generating verification codes."""
    
    @property
    def emoji_indices(self) -> List[int]: ...  # Actually returns [u8; 7] but typing as List[int]
    @property
    def decimals(self) -> Tuple[int, int, int]: ...

# PK Encryption (Public Key Encryption)
class Message:
    """A message encrypted using PK encryption."""
    
    def __init__(self, ciphertext: bytes, mac: bytes, ephemeral_key: bytes) -> None: ...
    @classmethod
    def from_base64(cls, ciphertext: str, mac: str, ephemeral_key: str) -> Message: ...
    def to_base64(self) -> Tuple[str, str, str]: ...
    
    ciphertext: bytes
    mac: bytes
    ephemeral_key: bytes

class PkDecryption:
    """PK decryption object."""
    
    def __init__(self) -> None: ...
    @classmethod
    def from_key(cls, key: Curve25519SecretKey) -> PkDecryption: ...
    @property
    def public_key(self) -> Curve25519PublicKey: ...
    def decrypt(self, message: Message) -> bytes: ...

class PkEncryption:
    """PK encryption object."""
    
    @classmethod
    def from_key(cls, key: Curve25519PublicKey) -> PkEncryption: ...
    def encrypt(self, message: bytes) -> Message: ... 


import nox_uv
from nox import Session, options

options.default_venv_backend = "uv"

@nox_uv.session(
    python=["3.10", "3.11", "3.12", "3.13", "3.14"],
    uv_groups=["tests"],
)
def test(session: Session):
    session.run("pytest")



import pytest
from base64 import b64encode
from hypothesis import given
from vodozemac import (
    Message,
    Curve25519PublicKey,
    Curve25519SecretKey,
    PkDecodeException,
    PkDecryption,
    PkEncryption,
)


@pytest.fixture(scope="module")
def pk_decryption() -> PkDecryption:
    return PkDecryption()

@pytest.fixture(scope="module")
def pk_encryption(pk_decryption: PkDecryption) -> PkEncryption:
    return PkEncryption.from_key(pk_decryption.public_key)

@pytest.fixture(scope="module")
def secret_key() -> Curve25519SecretKey:
    return Curve25519SecretKey()

@given(cleartext=...)
def test_round_trip(pk_decryption: PkDecryption, pk_encryption: PkEncryption, cleartext: bytes):
    assert cleartext == pk_decryption.decrypt(pk_encryption.encrypt(cleartext))

@given(cleartext=...)
def test_wrong_key(pk_decryption: PkDecryption, pk_encryption: PkEncryption, cleartext: bytes):
    with pytest.raises(PkDecodeException, match="MAC tag mismatch"):
        PkDecryption().decrypt(pk_encryption.encrypt(cleartext))

@given(cleartext=...)
def test_serialized_keys(secret_key: Curve25519SecretKey, cleartext: bytes):
    secret_key_bytes = secret_key.to_bytes()
    public_key_bytes = secret_key.public_key().to_bytes()

    d = PkDecryption.from_key(Curve25519SecretKey.from_bytes(secret_key_bytes))
    e = PkEncryption.from_key(Curve25519PublicKey.from_bytes(public_key_bytes))

    assert cleartext == d.decrypt(e.encrypt(cleartext))

@given(cleartext=...)
def test_encrypt_message_attr(cleartext: bytes):
    """Test that the Message object has accessible Python attributes (mac, ciphertext, ephemeral_key)."""
    decryption = PkDecryption()
    encryption = PkEncryption.from_key(decryption.public_key)

    message = encryption.encrypt(cleartext)

    assert message.mac is not None
    assert message.ciphertext is not None
    assert message.ephemeral_key is not None


def test_message_from_invalid_base64():
    """Test that invalid base64 input raises PkDecodeException."""
    # Test invalid ciphertext
    with pytest.raises(PkDecodeException, match="Invalid symbol"):
        Message.from_base64(
            "not-valid-base64!@#",  # Invalid base64 for ciphertext
            b64encode(b"some_mac").decode(),  # Valid base64
            b64encode(b"some_key").decode()   # Valid base64
        )

    # Test invalid mac
    with pytest.raises(PkDecodeException, match="Invalid symbol"):
        Message.from_base64(
            b64encode(b"some_text").decode(),
            "not-valid-base64!@#",  # Invalid base64 for mac
            b64encode(b"some_key").decode()
        )

    # Test invalid ephemeral key
    with pytest.raises(PkDecodeException, match="Invalid symbol"):
        Message.from_base64(
            b64encode(b"some_text").decode(),
            b64encode(b"some_mac").decode(),
            "not-valid-base64!@#"  # Invalid base64 for ephemeral key
        )



import pytest
from hypothesis import given
from vodozemac import (
    GroupSession,
    InboundGroupSession,
    MegolmDecryptionException,
    PickleException,
)

@pytest.fixture(scope="module")
def group_session() -> GroupSession:
    return GroupSession()

@pytest.fixture(scope="module")
def inbound_group_session(group_session: GroupSession) -> InboundGroupSession:
    return InboundGroupSession(group_session.session_key)


def test_create(group_session: GroupSession, inbound_group_session: InboundGroupSession):
    assert isinstance(group_session.session_id, str)
    assert isinstance(group_session.message_index, int)
    assert group_session.message_index == 0

    assert isinstance(inbound_group_session.first_known_index, int)
    assert inbound_group_session.first_known_index == 0

    assert group_session.session_id == inbound_group_session.session_id

def test_outbound_pickle(group_session: GroupSession, pickle_key: bytes):
    pickle = group_session.pickle(pickle_key)
    unpickled = GroupSession.from_pickle(pickle, pickle_key)

    assert group_session.session_id == unpickled.session_id

def test_outbound_pickle_fail(group_session: GroupSession, pickle_key: bytes):
    wrong_pickle_key = b"It's a secret to everybody 12345"
    pickle = group_session.pickle(wrong_pickle_key)

    with pytest.raises(ValueError):
        GroupSession.from_pickle(pickle, pickle_key)

@pytest.mark.parametrize("cls", (GroupSession, InboundGroupSession))
def test_invalid_pickle(cls: type, pickle_key: bytes):
    with pytest.raises(PickleException):
        cls.from_pickle("", pickle_key)


def test_inbound_create(inbound_group_session: InboundGroupSession, pickle_key: bytes):
    pickle = inbound_group_session.pickle(pickle_key)
    unpickled = InboundGroupSession.from_pickle(pickle, pickle_key)
    assert unpickled.session_id == inbound_group_session.session_id

@given(message1=..., message2=...)
def test_encrypt_twice(group_session: GroupSession, inbound_group_session: InboundGroupSession, message1: bytes, message2: bytes):
    decrypted1 = inbound_group_session.decrypt(group_session.encrypt(message1))
    assert decrypted1.plaintext == message1

    decrypted2 = inbound_group_session.decrypt(group_session.encrypt(message2))
    assert decrypted2.plaintext == message2

    assert decrypted2.message_index == decrypted1.message_index + 1

def test_decrypt_failure(inbound_group_session: InboundGroupSession):
    wrong_group_session = GroupSession()
    with pytest.raises(MegolmDecryptionException):
        inbound_group_session.decrypt(wrong_group_session.encrypt(b"Test"))


@given(message=...)
def test_inbound_export(group_session: GroupSession, inbound_group_session: InboundGroupSession, message: bytes):
    imported = InboundGroupSession.import_session(
        session_key=inbound_group_session.export_at(
            index=inbound_group_session.first_known_index
        )
    )
    index = group_session.message_index
    decrypted = imported.decrypt(group_session.encrypt(message))

    assert decrypted.plaintext == message
    assert decrypted.message_index == index

def test_outbound_clear():
    session = GroupSession()
    del session

def test_inbound_clear():
    outbound = GroupSession()
    inbound = InboundGroupSession(outbound.session_key)
    del inbound
